@@ -102,7 +102,7 @@ lsm_dom_node_child_list_get_length (LsmDomNodeList *list)
 	return length;
 }
 
-LsmDomNodeList *
+static LsmDomNodeList *
 lsm_dom_node_child_list_new (LsmDomNode *parent_node)
 {
 	LsmDomNodeChildList *list;
@@ -159,9 +159,10 @@ static GObjectClass *parent_class = NULL;
 /**
  * lsm_dom_node_get_node_name:
  * @self: a #LsmDomNode
- * Return value: the node name.
  *
  * Gets the node name.
+ *
+ * Return value: the node name.
  */
 
 const char*
@@ -180,9 +181,10 @@ lsm_dom_node_get_node_name (LsmDomNode* self)
 /**
  * lsm_dom_node_get_node_value:
  * @self: a #LsmDomNode
- * Return value: the node value.
  *
  * Gets the node value.
+ *
+ * Return value: the node value.
  */
 
 const char*
@@ -225,6 +227,7 @@ LsmDomNodeType lsm_dom_node_get_node_type (LsmDomNode* self)
 /**
  * lsm_dom_node_get_parent_node:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node parent
  */
 
@@ -239,6 +242,7 @@ lsm_dom_node_get_parent_node (LsmDomNode* self)
 /**
  * lsm_dom_node_get_child_nodes:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node child list
  */
 
@@ -262,6 +266,7 @@ lsm_dom_node_get_child_nodes (LsmDomNode* self)
 /**
  * lsm_dom_node_get_first_child:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node first child
  */
 
@@ -276,6 +281,7 @@ lsm_dom_node_get_first_child (LsmDomNode* self)
 /**
  * lsm_dom_node_get_last_child:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node last child
  */
 
@@ -290,6 +296,7 @@ lsm_dom_node_get_last_child (LsmDomNode* self)
 /**
  * lsm_dom_node_get_previous_sibling:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node previous sibling
  */
 
@@ -304,6 +311,7 @@ lsm_dom_node_get_previous_sibling (LsmDomNode* self)
 /**
  * lsm_dom_node_get_next_sibling:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node next sibling
  */
 
@@ -326,22 +334,16 @@ lsm_dom_node_get_attributes (LsmDomNode* self)
 /**
  * lsm_dom_node_get_owner_document:
  * @self: a #LsmDomNode
+ *
  * Returns: (transfer none): node owner document
  */
 
 LsmDomDocument *
 lsm_dom_node_get_owner_document (LsmDomNode* self)
 {
-	LsmDomNode *parent;
-
 	g_return_val_if_fail (LSM_IS_DOM_NODE (self), NULL);
 
-	for (parent = self;
-	     parent != NULL &&
-	     !LSM_IS_DOM_DOCUMENT (parent);
-	     parent = parent->parent_node);
-
-	return LSM_DOM_DOCUMENT (parent);
+	return self->owner_document;
 }
 
 /**
@@ -369,7 +371,7 @@ lsm_dom_node_insert_before (LsmDomNode* self, LsmDomNode* new_child, LsmDomNode*
 	LsmDomNodeClass *node_class;
 
 	if (ref_child == NULL)
-		lsm_dom_node_append_child (self, new_child);
+		return lsm_dom_node_append_child (self, new_child);
 
 	g_return_val_if_fail (LSM_IS_DOM_NODE (new_child), NULL);
 
@@ -382,6 +384,12 @@ lsm_dom_node_insert_before (LsmDomNode* self, LsmDomNode* new_child, LsmDomNode*
 		return NULL;
 	}
 
+	if (self->owner_document != new_child->owner_document &&
+	    self->owner_document != NULL) {
+		g_object_unref (new_child);
+		return NULL;
+	}
+
 	if (!LSM_IS_DOM_NODE (ref_child)) {
 		g_critical ("%s: ref_child is not a LsmDomNode", G_STRFUNC);
 		g_object_unref (new_child);
@@ -480,6 +488,13 @@ lsm_dom_node_replace_child (LsmDomNode* self, LsmDomNode* new_child, LsmDomNode*
 		return NULL;
 	}
 
+	if (self->owner_document != new_child->owner_document &&
+	    self->owner_document != NULL) {
+		g_object_unref (new_child);
+		g_object_unref (old_child);
+		return NULL;
+	}
+
 	if (old_child->parent_node != self) {
 		g_object_unref (new_child);
 		g_object_unref (old_child);
@@ -584,6 +599,12 @@ lsm_dom_node_append_child (LsmDomNode* self, LsmDomNode* new_child)
 		return NULL;
 	}
 
+	if (self->owner_document != new_child->owner_document &&
+	    self->owner_document != NULL) {
+		g_object_unref (new_child);
+		return NULL;
+	}
+
 	if (new_child->parent_node != NULL)
 		lsm_dom_node_remove_child (self, new_child);
 
