@@ -103,6 +103,71 @@ const LsmTraitClass lsm_svg_enable_background_trait_class = {
 	.to_string = lsm_svg_enable_background_trait_to_string
 };
 
+static gboolean
+lsm_svg_vector_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgVector *vector = (LsmSvgVector *) abstract_trait;
+	unsigned int n_values = 1;
+	gboolean success = FALSE;
+	char *iter = (char *) string;
+
+	g_free (vector->values);
+	vector->n_values = 0;
+	vector->values = NULL;
+
+	while (*iter != '\0') {
+		if (*iter == ',' ||
+		    *iter == ' ') {
+			n_values++;
+			do {
+				iter++;
+			} while (*iter == ',' ||
+				 *iter == ' ');
+		} else
+			iter++;
+	}
+
+	vector->values = g_new (double, n_values);
+	vector->n_values = n_values;
+
+	iter = (char *)string;
+	n_values = lsm_str_parse_double_list (&iter, vector->n_values, vector->values);
+	success = n_values == vector->n_values;
+
+	if (!success) {
+		g_free (vector->values);
+		vector->values = NULL;
+		vector->n_values = 0;
+	}
+
+	return success;
+}
+
+static char *
+lsm_svg_vector_trait_to_string (LsmTrait *abstract_trait)
+{
+	g_assert_not_reached ();
+
+	return NULL;
+}
+
+static void
+lsm_svg_vector_trait_finalize (LsmTrait *abstract_trait)
+{
+	LsmSvgVector *svg_vector = (LsmSvgVector *) abstract_trait;
+
+	g_free (svg_vector->values);
+	svg_vector->n_values = 0;
+	svg_vector->values = NULL;
+}
+
+const LsmTraitClass lsm_svg_vector_trait_class = {
+	.size = sizeof (LsmSvgVector),
+	.from_string = lsm_svg_vector_trait_from_string,
+	.to_string = lsm_svg_vector_trait_to_string,
+	.finalize = lsm_svg_vector_trait_finalize
+};
+
 static gboolean
 lsm_svg_length_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
@@ -935,6 +1000,30 @@ const LsmTraitClass lsm_svg_color_trait_class = {
 	.to_string = lsm_svg_color_trait_to_string
 };
 
+static gboolean
+lsm_svg_color_filter_type_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgColorFilterType *trait = (LsmSvgColorFilterType *) abstract_trait;
+
+	*trait = lsm_svg_color_filter_type_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_color_filter_type_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgColorFilterType *trait = (LsmSvgColorFilterType *) abstract_trait;
+
+	return g_strdup (lsm_svg_color_filter_type_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_color_filter_type_trait_class = {
+	.size = sizeof (LsmSvgColorFilterType),
+	.from_string = lsm_svg_color_filter_type_trait_from_string,
+	.to_string = lsm_svg_color_filter_type_trait_to_string
+};
+
 static gboolean
 lsm_svg_marker_units_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
@@ -1182,6 +1271,47 @@ const LsmTraitClass lsm_svg_one_or_two_double_trait_class = {
 	.to_string = lsm_svg_one_or_two_double_trait_to_string
 };
 
+static gboolean
+lsm_svg_one_or_two_integer_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgOneOrTwoInteger *trait = (LsmSvgOneOrTwoInteger *) abstract_trait;
+	char *end_ptr;
+
+	trait->a = g_ascii_strtoll (string, &end_ptr, 10);
+	if (end_ptr == string) {
+		trait->a = 0;
+		trait->b = 0;
+		return FALSE;
+	}
+
+	string = end_ptr;
+
+	lsm_str_skip_spaces (&string);
+
+	if (string[0] == '\0') {
+		trait->b = trait->a;
+		return TRUE;
+	}
+
+	trait->b = g_ascii_strtoll (string, &end_ptr, 10);
+
+	return TRUE;
+}
+
+static char *
+lsm_svg_one_or_two_integer_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgOneOrTwoInteger *trait = (LsmSvgOneOrTwoInteger *) abstract_trait;
+
+	return g_strdup_printf ("%d %d", trait->a, trait->b);
+}
+
+const LsmTraitClass lsm_svg_one_or_two_integer_trait_class = {
+	.size = sizeof (double),
+	.from_string = lsm_svg_one_or_two_integer_trait_from_string,
+	.to_string = lsm_svg_one_or_two_integer_trait_to_string
+};
+
 static gboolean
 lsm_svg_overflow_trait_from_string (LsmTrait *abstract_trait, char *string)
 {
@@ -1230,3 +1360,122 @@ const LsmTraitClass lsm_svg_writing_mode_trait_class = {
 	.to_string = lsm_svg_writing_mode_trait_to_string
 };
 
+static gboolean
+lsm_svg_morphology_operator_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgMorphologyOperator *trait = (LsmSvgMorphologyOperator *) abstract_trait;
+
+	*trait = lsm_svg_morphology_operator_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_morphology_operator_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgMorphologyOperator *trait = (LsmSvgMorphologyOperator *) abstract_trait;
+
+	return g_strdup (lsm_svg_morphology_operator_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_morphology_operator_trait_class = {
+	.size = sizeof (LsmSvgMorphologyOperator),
+	.from_string = lsm_svg_morphology_operator_trait_from_string,
+	.to_string = lsm_svg_morphology_operator_trait_to_string
+};
+
+static gboolean
+lsm_svg_edge_mode_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgEdgeMode *trait = (LsmSvgEdgeMode *) abstract_trait;
+
+	*trait = lsm_svg_edge_mode_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_edge_mode_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgEdgeMode *trait = (LsmSvgEdgeMode *) abstract_trait;
+
+	return g_strdup (lsm_svg_edge_mode_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_edge_mode_trait_class = {
+	.size = sizeof (LsmSvgEdgeMode),
+	.from_string = lsm_svg_edge_mode_trait_from_string,
+	.to_string = lsm_svg_edge_mode_trait_to_string
+};
+
+static gboolean
+lsm_svg_stitch_tiles_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgStitchTiles *trait = (LsmSvgStitchTiles *) abstract_trait;
+
+	*trait = lsm_svg_stitch_tiles_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_stitch_tiles_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgStitchTiles *trait = (LsmSvgStitchTiles *) abstract_trait;
+
+	return g_strdup (lsm_svg_stitch_tiles_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_stitch_tiles_trait_class = {
+	.size = sizeof (LsmSvgStitchTiles),
+	.from_string = lsm_svg_stitch_tiles_trait_from_string,
+	.to_string = lsm_svg_stitch_tiles_trait_to_string
+};
+
+static gboolean
+lsm_svg_turbulence_type_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgTurbulenceType *trait = (LsmSvgTurbulenceType *) abstract_trait;
+
+	*trait = lsm_svg_turbulence_type_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_turbulence_type_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgTurbulenceType *trait = (LsmSvgTurbulenceType *) abstract_trait;
+
+	return g_strdup (lsm_svg_turbulence_type_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_turbulence_type_trait_class = {
+	.size = sizeof (LsmSvgTurbulenceType),
+	.from_string = lsm_svg_turbulence_type_trait_from_string,
+	.to_string = lsm_svg_turbulence_type_trait_to_string
+};
+
+static gboolean
+lsm_svg_channel_selector_trait_from_string (LsmTrait *abstract_trait, char *string)
+{
+	LsmSvgChannelSelector *trait = (LsmSvgChannelSelector *) abstract_trait;
+
+	*trait = lsm_svg_channel_selector_from_string (string);
+
+	return *trait >= 0;
+}
+
+static char *
+lsm_svg_channel_selector_trait_to_string (LsmTrait *abstract_trait)
+{
+	LsmSvgChannelSelector *trait = (LsmSvgChannelSelector *) abstract_trait;
+
+	return g_strdup (lsm_svg_channel_selector_to_string (*trait));
+}
+
+const LsmTraitClass lsm_svg_channel_selector_trait_class = {
+	.size = sizeof (LsmSvgChannelSelector),
+	.from_string = lsm_svg_channel_selector_trait_from_string,
+	.to_string = lsm_svg_channel_selector_trait_to_string
+};
