@@ -331,11 +331,20 @@ lsm_svg_view_create_surface_pattern (LsmSvgView *view,
 	g_return_val_if_fail (view->pattern_data != NULL, FALSE);
 	g_return_val_if_fail (view->dom_view.cairo == NULL, FALSE);
 
+	/* Calculate a scale that will allow us to create a surface pattern
+	 * for which a pixel will also be roughly a pixel on the final
+	 * image */
+
 	x1 = viewport->x;
 	y1 = viewport->y;
 	x2 = viewport->x + viewport->width;
 	y2 = viewport->y;
 
+	if (pattern_matrix != NULL) {
+		lsm_svg_matrix_transform_point (pattern_matrix, &x1, &y1);
+		lsm_svg_matrix_transform_point (pattern_matrix, &x2, &y2);
+	}
+
 	cairo_user_to_device (view->pattern_data->old_cairo, &x1, &y1);
 	cairo_user_to_device (view->pattern_data->old_cairo, &x2, &y2);
 
@@ -344,14 +353,18 @@ lsm_svg_view_create_surface_pattern (LsmSvgView *view,
 	x2 = viewport->x;
 	y2 = viewport->y + viewport->height;
 
+	if (pattern_matrix != NULL) {
+		lsm_svg_matrix_transform_point (pattern_matrix, &x2, &y2);
+	}
+
 	cairo_user_to_device (view->pattern_data->old_cairo, &x2, &y2);
 
 	device_height = sqrt ((x1 - x2)*(x1 - x2) + (y1 - y2)*(y1 - y2));
 
-	if (surface_type == LSM_SVG_VIEW_SURFACE_TYPE_IMAGE) {
-		device_height = ceil (device_height);
-		device_width = ceil (device_width);
-	}
+	/* Create new surface with integer size */
+
+	device_height = ceil (device_height);
+	device_width = ceil (device_width);
 
 	x_scale = device_width / viewport->width;
 	y_scale = device_height / viewport->height;
@@ -1761,23 +1774,25 @@ lsm_svg_view_pop_mask (LsmSvgView *view)
 		cairo_pop_group_to_source (cairo);
 		if (view->pattern_data->pattern != NULL) {
 			cairo_surface_t *surface;
-			int width, height, row, i, stride;
-			unsigned char *pixels;
 
-			cairo_pattern_get_surface (view->pattern_data->pattern, &surface);
-			pixels = cairo_image_surface_get_data (surface);
-			height = cairo_image_surface_get_height (surface);
-			width = cairo_image_surface_get_width (surface);
-			stride = cairo_image_surface_get_stride (surface);
+			if (cairo_pattern_get_surface (view->pattern_data->pattern, &surface) == CAIRO_STATUS_SUCCESS) {
+				int width, height, row, i, stride;
+				unsigned char *pixels;
 
-			for (row = 0; row < height; row++) {
-				guint8 *row_data = (pixels + (row * stride));
-				for (i = 0; i < width; i++) {
-					guint32 *pixel = (guint32 *) row_data + i;
-					*pixel = ((((*pixel & 0x00ff0000) >> 16) * 13817 +
-						   ((*pixel & 0x0000ff00) >> 8) * 46518 +
-						   ((*pixel & 0x000000ff)) * 4688) * 0xff
-						  /* * opacity */);
+				pixels = cairo_image_surface_get_data (surface);
+				height = cairo_image_surface_get_height (surface);
+				width = cairo_image_surface_get_width (surface);
+				stride = cairo_image_surface_get_stride (surface);
+
+				for (row = 0; row < height; row++) {
+					guint8 *row_data = (pixels + (row * stride));
+					for (i = 0; i < width; i++) {
+						guint32 *pixel = (guint32 *) row_data + i;
+						*pixel = ((((*pixel & 0x00ff0000) >> 16) * 13817 +
+							   ((*pixel & 0x0000ff00) >> 8) * 46518 +
+							   ((*pixel & 0x000000ff)) * 4688) * 0xff
+							  /* * opacity */);
+					}
 				}
 			}
 
@@ -1868,58 +1883,58 @@ lsm_svg_view_pop_filter (LsmSvgView *view)
 
 	if (LSM_IS_SVG_FILTER_ELEMENT (filter_element) &&
 	    view->pattern_data->pattern != NULL) {
-		cairo_matrix_t matrix;
-		LsmBox subregion;
+		if (cairo_pattern_get_surface (view->pattern_data->pattern, &surface) == CAIRO_STATUS_SUCCESS) {
+			cairo_matrix_t matrix;
+			LsmBox subregion;
 
-		view->filter_surfaces = NULL;
+			view->filter_surfaces = NULL;
 
-		cairo_pattern_get_surface (view->pattern_data->pattern, &surface);
+			subregion.x = 0;
+			subregion.y = 0;
+			subregion.width = cairo_image_surface_get_width (surface);
+			subregion.height = cairo_image_surface_get_height (surface);
 
-		subregion.x = 0;
-		subregion.y = 0;
-		subregion.width = cairo_image_surface_get_width (surface);
-		subregion.height = cairo_image_surface_get_height (surface);
+			filter_surface = lsm_svg_filter_surface_new_with_content ("SourceGraphic", surface, &subregion);
+			cairo_pattern_get_matrix (view->pattern_data->pattern, &matrix);
 
-		filter_surface = lsm_svg_filter_surface_new_with_content ("SourceGraphic", surface, &subregion);
-		cairo_pattern_get_matrix (view->pattern_data->pattern, &matrix);
+			view->filter_surfaces = g_slist_prepend (view->filter_surfaces, filter_surface);
 
-		view->filter_surfaces = g_slist_prepend (view->filter_surfaces, filter_surface);
+			lsm_svg_element_force_render (filter_element, view);
 
-		lsm_svg_element_force_render (filter_element, view);
+			if (view->debug_filter) {
+				GSList *iter;
+				char *filename;
+				static int count = 0;
 
-		if (view->debug_filter) {
-			GSList *iter;
-			char *filename;
-			static int count = 0;
+				for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
+					LsmSvgFilterSurface *surface = iter->data;
 
-			for (iter = view->filter_surfaces; iter != NULL; iter = iter->next) {
-				LsmSvgFilterSurface *surface = iter->data;
-
-				filename = g_strdup_printf ("filter-%04d-%s-%s.png", count++,
-							    view->style->filter->value,
-							    lsm_svg_filter_surface_get_name (surface));
-				cairo_surface_write_to_png (lsm_svg_filter_surface_get_cairo_surface (surface), filename);
-				g_free (filename);
+					filename = g_strdup_printf ("filter-%04d-%s-%s.png", count++,
+							view->style->filter->value,
+							lsm_svg_filter_surface_get_name (surface));
+					cairo_surface_write_to_png (lsm_svg_filter_surface_get_cairo_surface (surface), filename);
+					g_free (filename);
+				}
 			}
+
+			if (view->filter_surfaces->next != NULL) {
+				cairo_pattern_t *pattern;
+				cairo_surface_t *surface;
+
+				surface = lsm_svg_filter_surface_get_cairo_surface (view->filter_surfaces->data);
+				pattern = cairo_pattern_create_for_surface (surface);
+				cairo_pattern_set_extend (pattern, CAIRO_EXTEND_NONE);
+				cairo_pattern_set_matrix (pattern, &matrix);
+				cairo_set_source (view->pattern_data->old_cairo, pattern);
+				cairo_pattern_destroy (pattern);
+				cairo_paint_with_alpha (view->pattern_data->old_cairo, view->style->opacity->value);
+			}
+
+			for (iter = view->filter_surfaces; iter != NULL; iter = iter->next)
+				lsm_svg_filter_surface_unref (iter->data);
+			g_slist_free (view->filter_surfaces);
+			view->filter_surfaces = NULL;
 		}
-
-		if (view->filter_surfaces->next != NULL) {
-			cairo_pattern_t *pattern;
-			cairo_surface_t *surface;
-
-			surface = lsm_svg_filter_surface_get_cairo_surface (view->filter_surfaces->data);
-			pattern = cairo_pattern_create_for_surface (surface);
-			cairo_pattern_set_extend (pattern, CAIRO_EXTEND_NONE);
-			cairo_pattern_set_matrix (pattern, &matrix);
-			cairo_set_source (view->pattern_data->old_cairo, pattern);
-			cairo_pattern_destroy (pattern);
-			cairo_paint_with_alpha (view->pattern_data->old_cairo, view->style->opacity->value);
-		}
-
-		for (iter = view->filter_surfaces; iter != NULL; iter = iter->next)
-			lsm_svg_filter_surface_unref (iter->data);
-		g_slist_free (view->filter_surfaces);
-		view->filter_surfaces = NULL;
 	}
 
 	_end_pattern (view);
@@ -2095,10 +2110,30 @@ lsm_svg_view_apply_flood (LsmSvgView *view, const char *output, const LsmBox *su
 		        subregion_px.x, subregion_px.y);
 
 	lsm_svg_filter_surface_flood (output_surface,
-				  view->style->flood_color->value.red,
-				  view->style->flood_color->value.green,
-				  view->style->flood_color->value.blue,
-				  view->style->flood_opacity->value);
+				      view->style->flood_color->value.red,
+				      view->style->flood_color->value.green,
+				      view->style->flood_color->value.blue,
+				      view->style->flood_opacity->value);
+}
+
+void
+lsm_svg_view_apply_image (LsmSvgView *view, const char *output, const LsmBox *subregion,
+			  GdkPixbuf *pixbuf, LsmSvgPreserveAspectRatio preserve_aspect_ratio)
+{
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmBox subregion_px;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, NULL);
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_log_render ("[SvgView::apply_image]");
+
+	lsm_svg_filter_surface_image (output_surface, pixbuf, preserve_aspect_ratio);
 }
 
 void
@@ -2131,7 +2166,7 @@ lsm_svg_view_apply_gaussian_blur (LsmSvgView *view, const char *input, const cha
 	lsm_log_render ("[SvgView::apply_gaussian_blur] %g px,%g px",
 			std_x, std_y);
 
-	lsm_svg_filter_surface_fast_blur (input_surface, output_surface, std_x, std_y);
+	lsm_svg_filter_surface_blur (input_surface, output_surface, std_x, std_y);
 }
 
 void
@@ -2163,6 +2198,172 @@ lsm_svg_view_apply_offset (LsmSvgView *view, const char *input, const char *outp
 	lsm_svg_filter_surface_offset (input_surface, output_surface, dx, dy);
 }
 
+void
+lsm_svg_view_apply_color_matrix (LsmSvgView *view, const char *input, const char *output,
+				 const LsmBox *subregion, LsmSvgColorFilterType type,
+				 unsigned int n_values, const double *values)
+{
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_color_matrix] Input '%s' not found", input);
+		return;
+	}
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_svg_filter_surface_color_matrix (input_surface, output_surface, type, n_values, values);
+}
+
+void
+lsm_svg_view_apply_displacement_map (LsmSvgView *view, const char *input_1, const char *input_2, const char *output,
+				     const LsmBox *subregion,
+				     double scale, LsmSvgChannelSelector x_channel_selector, LsmSvgChannelSelector y_channel_selector)
+{
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_1_surface;
+	LsmSvgFilterSurface *input_2_surface;
+	LsmBox subregion_px;
+	cairo_matrix_t transform;
+	double x_scale, y_scale;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_1_surface = _get_filter_surface (view, input_1);
+	input_2_surface = _get_filter_surface (view, input_2);
+
+	if (input_1_surface == NULL || input_2_surface == NULL) {
+		lsm_warning_render ("[SvgView::apply_displacement_map] Inputs '%s' or '%s' not found", input_1, input_2);
+		return;
+	}
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_1_surface, &subregion_px);
+
+	cairo_get_matrix (view->dom_view.cairo, &transform);
+
+	x_scale = transform.xx * scale;
+	y_scale = transform.yy * scale;
+
+	lsm_svg_filter_surface_displacement_map (input_1_surface, input_2_surface, output_surface, x_scale, y_scale,
+						 x_channel_selector, y_channel_selector);
+}
+
+void 
+lsm_svg_view_apply_morphology (LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
+			       LsmSvgMorphologyOperator op, double radius)
+{
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
+	double rx, ry;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_morphoogy] Input '%s' not found", input);
+		return;
+	}
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	rx = radius;
+	ry = radius;
+	cairo_user_to_device_distance (view->dom_view.cairo, &rx, &ry);
+
+	lsm_svg_filter_surface_morphology (input_surface, output_surface, op, rx, ry);
+}
+
+void
+lsm_svg_view_apply_convolve_matrix (LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
+				    unsigned x_order, unsigned y_order, unsigned n_values, double *values,
+				    double divisor, double bias, int target_x, int target_y,
+				    LsmSvgEdgeMode edge_mode, gboolean preserve_alpha)
+{
+	LsmSvgFilterSurface *input_surface;
+	LsmSvgFilterSurface *output_surface;
+	LsmBox subregion_px;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, input);
+
+	if (input_surface == NULL) {
+		lsm_debug_render ("[SvgView::apply_color_matrix] Input '%s' not found", input);
+		return;
+	}
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_svg_filter_surface_convolve_matrix (input_surface, output_surface, x_order, y_order, n_values, values,
+						divisor, bias, target_x, target_y, edge_mode, preserve_alpha);
+}
+
+void 
+lsm_svg_view_apply_specular_lighting (LsmSvgView *view, const char *output, const LsmBox *subregion,
+				      double surface_scale, double specular_constant, double specular_exponent,
+				      double dx, double dy)
+{
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmBox subregion_px;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, NULL);
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_log_render ("[SvgView::apply_specular_lighting] subregion %gx%g px at %g,%g px",
+			subregion_px.width, subregion_px.height,
+			subregion_px.x, subregion_px.y);
+
+	cairo_user_to_device_distance (view->dom_view.cairo, &dx, &dy);
+
+	lsm_svg_filter_surface_specular_lighting (output_surface, surface_scale, specular_constant, specular_exponent, dx, dy);
+}
+
+void
+lsm_svg_view_apply_turbulence (LsmSvgView *view, const char *output, const LsmBox *subregion,
+			       double base_frequency_x, double base_frequency_y,
+			       int n_octaves, double seed,
+			       LsmSvgStitchTiles stitch_tiles,
+			       LsmSvgTurbulenceType type)
+{
+	LsmSvgFilterSurface *output_surface;
+	LsmSvgFilterSurface *input_surface;
+	LsmBox subregion_px;
+	cairo_matrix_t transform;
+
+	g_return_if_fail (LSM_IS_SVG_VIEW (view));
+
+	input_surface = _get_filter_surface (view, NULL);
+
+	lsm_cairo_box_user_to_device (view->dom_view.cairo, &subregion_px, subregion);
+	output_surface = _create_filter_surface (view, output, input_surface, &subregion_px);
+
+	lsm_log_render ("[SvgView::apply_turbulence] subregion %gx%g px at %g,%g px",
+			subregion_px.width, subregion_px.height,
+			subregion_px.x, subregion_px.y);
+
+	cairo_get_matrix (view->dom_view.cairo, &transform);
+
+	lsm_svg_filter_surface_turbulence (output_surface, base_frequency_x, base_frequency_y, n_octaves, seed, stitch_tiles, type,
+					   &transform);
+}
+
 void
 lsm_svg_view_apply_merge (LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion)
 {
@@ -2226,6 +2427,18 @@ lsm_svg_view_pop_element (LsmSvgView *view)
 	view->element_stack = g_slist_delete_link (view->element_stack, view->element_stack);
 }
 
+LsmSvgElement *
+lsm_svg_view_get_referencing_element (LsmSvgView *view)
+{
+	g_return_val_if_fail (LSM_IS_SVG_VIEW (view), NULL);
+	
+	if (view->element_stack == NULL ||
+	    view->element_stack->next == NULL)
+		return NULL;
+	
+	return view->element_stack->next->data;
+}
+
 static gboolean
 lsm_svg_view_circular_reference_check (LsmSvgView *view, LsmSvgElement *element)
 {
