@@ -155,6 +155,7 @@ void 		lsm_svg_view_pop_matrix			(LsmSvgView *view);
 
 void		lsm_svg_view_push_element		(LsmSvgView *view, const LsmSvgElement *element);
 void		lsm_svg_view_pop_element		(LsmSvgView *view);
+LsmSvgElement * lsm_svg_view_get_referencing_element 	(LsmSvgView *view);
 
 void		lsm_svg_view_push_style			(LsmSvgView *view, LsmSvgStyle *style);
 void		lsm_svg_view_pop_style			(LsmSvgView *view);
@@ -171,8 +172,31 @@ void		lsm_svg_view_apply_gaussian_blur 	(LsmSvgView *view, const char *input, co
 							 double std_x, double std_y);
 void 		lsm_svg_view_apply_offset 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
 							 double dx, double dy);
+void 		lsm_svg_view_apply_color_matrix 	(LsmSvgView *view, const char *input, const char *output,
+							 const LsmBox *subregion, LsmSvgColorFilterType type,
+							 unsigned int n_values, const double *values);
+void		lsm_svg_view_apply_displacement_map 	(LsmSvgView *view, const char *input_1, const char *input_2, const char *output,
+							 const LsmBox *subregion, double scale,
+							 LsmSvgChannelSelector x_channel_selector,
+							 LsmSvgChannelSelector y_channel_selector);
 void 		lsm_svg_view_apply_merge 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion);
 void 		lsm_svg_view_apply_tile 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion);
+void		lsm_svg_view_apply_image 		(LsmSvgView *view, const char *output, const LsmBox *subregion,
+							 GdkPixbuf *pixbuf, LsmSvgPreserveAspectRatio preserve_aspect_ratio);
+void 		lsm_svg_view_apply_morphology 		(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
+							 LsmSvgMorphologyOperator op, double radius);
+void 		lsm_svg_view_apply_convolve_matrix 	(LsmSvgView *view, const char *input, const char *output, const LsmBox *subregion,
+							 unsigned a, unsigned b, unsigned n_values, double *values,
+							 double divisor, double bias, int target_x, int target_y,
+							 LsmSvgEdgeMode edge_mode, gboolean preserve_alpha);
+void 		lsm_svg_view_apply_specular_lighting 	(LsmSvgView *view, const char *output, const LsmBox *subregion,
+							 double surface_scale, double specular_constant, double specular_exponent,
+							 double dx, double dy);
+void		lsm_svg_view_apply_turbulence 		(LsmSvgView *view, const char *output, const LsmBox *subregion,
+							 double base_frequency_x, double base_frequency_y,
+							 int n_octaves, double seed,
+							 LsmSvgStitchTiles stitch_tiles,
+							 LsmSvgTurbulenceType type);
 
 G_END_DECLS
 
